//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Create a filename based on today's date, or other variables

#define DEBUG 0
#include"gd_tools.h"
#include"gd_nw.h"


#define pgVERS_ "V4.10"
#define pgTITL_ "Create a File Name based on Today's Date, Time, NIC, etc."
#define pgCOPR_ "1994"


#define PARLEN  256


void    d_more( void )
{
    fprintf( stderr, "\rPress a key to continue..." );
    while ( ! _getch() )
        ;
    fprintf( stderr, "\r                          \r" );
}


void    show_help( void )
{
    logon(1);
    printf(
            "Usage:\t%s mode template envvar batchfile\n\n"
            "mode\tis /DOW or /JUL or /WK or /YMD, etc:\n",
            get_global_program_name()
          );

    printf(
            "\t  /DOW = insert 1-char day of week at marker\n"
            "\t  /JUL = insert 3-char julian day at marker\n"
            "\t  /HM  = insert 4-char hour/minute number at marker\n"
            "\t  /HMS = insert 6-char hour/minute/second at marker\n"
            "\t  /HMST= insert 8-char hour/minute/second/tics at marker\n"
            "\t  /HR  = insert 2-char hour number at marker\n"
            "\t  /MIN = insert 2-char minute number at marker\n"
            "\t  /MTH = insert 2-char month number at marker\n"
            "\t  /NIC = insert rightmost 8 chars of NIC (network int. card) at marker\n"
            "\t  /RND = insert 8-char random number (0-99999999) at marker\n"
            "\t  /WK  = insert 2-char week number at marker\n"
            "\t  /YJUL= insert 2-char year + 3-char julian day at marker\n"
            "\t  /YM  = insert 2-char year + 2-char month number at marker\n"
            "\t  /YMD = insert 8-char date at marker\n"
            "\t  /YWK = insert 2-char year + 2-char week number at marker\n"
            "\t  /YY  = insert 2-char year at marker\n"
          );

    d_more();

    printf(
            "\t  \"marker\" is a \"*\"\n"
            "template  is the template to use for the filename to generate,\n"
            "\t  e.g. audit*.dat\n"
            "envvar    is the environment variable to create the filename in\n"
            "batchfile is the name of the batch file that sets envvar\n"
          );

    exit( BAD_EXIT );
}


void    main( int argc, char *argv[] )
{
    INT16       sys_day, sys_mth, sys_yr, sys_jul, sys_wk ;
    INT32       sys_ymd ;
    int         sys_yy  ;                       // last 2 digits of year
    int         sys_dow ;                       // day of week 1-7
    INT32       hhmmsstt, tics ;                // time vars.
    INT16       hh, mm, ss, tt ;                // time vars.
    UCHAR       aMode[PARLEN], aTempl[PARLEN], aT1[PARLEN], aT2[PARLEN], aEvar[PARLEN],
                aBatch[PARLEN], sNum[21] ;
    FILE        *oFh ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    if ( argc != 5 )
        show_help();

    st_cpy( aMode , (UCHAR *) argv[1], PARLEN );
    st_cpy( aTempl, (UCHAR *) argv[2], PARLEN );
    st_cpy( aEvar , (UCHAR *) argv[3], PARLEN );
    st_cpy( aBatch, (UCHAR *) argv[4], PARLEN );
    strupr( (char *) aMode  );
    strupr( (char *) aTempl );
    strupr( (char *) aEvar  );
    strupr( (char *) aBatch );

//  get today's date and other numbers based on the date
    dt_getsystemdate( &sys_ymd, &sys_yr, &sys_mth, &sys_day );
    sys_dow = dt_dow( sys_day, sys_mth, sys_yr );
    sys_wk  = dt_weeknumber( sys_ymd );
    sys_jul = JulianDay( sys_ymd );
    sys_yy  = sys_yr % 100 ;

//  ... and the time
    tm_getsystemtime( &hhmmsstt, &tics, &hh, &mm, &ss, &tt );

    if ( st_eq( aMode, "/DOW" ) || st_eq( aMode, "DOW" ) )  // day of week
    {
        sprintf( sNum, "%d", sys_dow );
    }
    else
    if ( st_eq( aMode, "/JUL" ) || st_eq( aMode, "JUL" ) )  // julian day
    {
        sprintf( sNum, "%03d", sys_jul );
    }
    else
    if ( st_eq( aMode, "/HM" ) || st_eq( aMode, "HM" ) )    // hours/minutes
    {
        sprintf( sNum, "%02d%02d", hh, mm );
    }
    else
    if ( st_eq( aMode, "/HMS" ) || st_eq( aMode, "HMS" ) )  // hours/minutes/secs
    {
        sprintf( sNum, "%02d%02d%02d", hh, mm, ss );
    }
    else
    if ( st_eq( aMode, "/HMST" ) || st_eq( aMode, "HMST" ) )// hours/minutes/secs/tics
    {
        sprintf( sNum, "%08lu", hhmmsstt );
    }
    else
    if ( st_eq( aMode, "/HR" ) || st_eq( aMode, "HR" ) )    // hours
    {
        sprintf( sNum, "%02d", hh );
    }
    else
    if ( st_eq( aMode, "/MIN" ) || st_eq( aMode, "MIN" ) )  // minutes
    {
        sprintf( sNum, "%02d", mm );
    }
    else
    if ( st_eq( aMode, "/MTH" ) || st_eq( aMode, "MTH" ) )  // mm
    {
        sprintf( sNum, "%02d", sys_mth );
    }
    else
    if ( st_eq( aMode, "/NIC" ) || st_eq( aMode, "NIC" ) )  // network card address
    {
        UCHAR   node_addr[13], username[49] ;
        nw_get_username( username );
        nw_get_station_address( node_addr );
        if ( strlen(node_addr) == 12 && strlen(username) )
            sprintf( sNum, "%s", &node_addr[4] );
        else
            sprintf( sNum, "%s", "$local" );
    }
    else
    if ( st_eq( aMode, "/RND" ) || st_eq( aMode, "RND" ) )  // random number
    {
        ULONG rnd ;
        RandomSeed();                           // seed random number generator
        rnd = random_ULONG( 0L, 99999999L );
        sprintf( sNum, "%08lu", rnd );
    }
    else
    if ( st_eq( aMode, "/WK"  ) || st_eq( aMode, "WK"  ) )  // week #
    {
        sprintf( sNum, "%02d", sys_wk );
    }
    else
    if ( st_eq( aMode, "/YJUL" ) || st_eq( aMode, "YJUL" ) )// yyjjj
    {
        sprintf( sNum, "%02d%03d", sys_yy, sys_jul );
    }
    else
    if ( st_eq( aMode, "/YM" ) || st_eq( aMode, "YM" ) )    // yymm
    {
        sprintf( sNum, "%02d%02d", sys_yy, sys_mth );
    }
    else
    if ( st_eq( aMode, "/YMD" ) || st_eq( aMode, "YMD" ) )  // yyyymmdd
    {
        sprintf( sNum, "%08ld", sys_ymd );
    }
    else
    if ( st_eq( aMode, "/YWK" ) || st_eq( aMode, "YWK" ) )  // yyww
    {
        sprintf( sNum, "%02d%02d", sys_yy, sys_wk );
    }
    else
    if ( st_eq( aMode, "/YY" ) || st_eq( aMode, "YY" ) )    // yy
    {
        sprintf( sNum, "%02d", sys_yy );
    }
    else
    {
        printf( "\nIncorrect MODE\a\n" );
        show_help();
    }

    SplitString( aTempl, "*", aT1, aT2 );
    st_trim( aT1 );
    st_trim( aT2 );
//  if ( (! strlen(aT1)) || (! strlen(aT2)) )   // if no marker, no file name
//      aT1[0] = aT2[0] = EOS ;

    fl_delete( aBatch );
    if ( ( oFh = fopen( aBatch, "w" ) ) == NULL )
    {
        fprintf( stderr, "\n\nCan't create batch file %s\a\n", aBatch );
        show_help();
    }
    fprintf( oFh, "@echo off\n rem apply mode %s to template %s\n set %s=%s%s%s\n",
                  aMode, aTempl, aEvar, aT1, sNum, aT2 );
    fclose ( oFh );

#if DEBUG
    printf( "yyyymmdd = %08ld %02d-%02d-%04d\n", sys_ymd, sys_day, sys_mth, sys_yr );
    printf( "julian   = %03d\n", sys_jul );
    printf( "dow      = %d\n", sys_dow );
    printf( "week #   = %02d\n", sys_wk );
#endif

    exit( GOOD_EXIT );
}
